@@ -24,6 +24,8 @@ utilities.boolean = function (value, default)
   if value == "true" then return true end
   if value == "no" then preferbool(); return false end
   if value == "yes" then preferbool(); return true end
+  if value == nil then return default end
+  SU.error("Expecting a boolean value but got '" .. value .. "'")
   return default
 end
 
@@ -40,6 +42,7 @@ utilities.error = function(message, bug)
 end
 
 utilities.warn = function(message, bug)
+  if SILE.quiet then return end
   io.stderr:write("\n! " .. message)
   if SILE.traceback or bug then
     io.stderr:write(" at:\n" .. SILE.traceStack:locationTrace())
@@ -53,7 +56,7 @@ utilities.warn = function(message, bug)
 end
 
 utilities.debugging = function (category)
-  return SILE.debugFlags.all or SILE.debugFlags[category]
+  return SILE.debugFlags.all and category ~= "profile" or SILE.debugFlags[category]
 end
 
 utilities.feq = function (lhs, rhs) -- Float point equal
@@ -103,14 +106,17 @@ utilities.deprecated = function (old, new, warnat, errorat, extra)
 end
 
 utilities.debug = function (category, ...)
+  if SILE.quiet then return end
   if utilities.debugging(category) then
     local inputs = table.pack(...)
     for i, input in ipairs(inputs) do
       if type(input) == "function" then
-        inputs[i] = input()
+        local status, output = pcall(input)
+        inputs[i] = status and output or SU.warn(("Output of %s debug function was an error: %s"):format(category, output))
       end
     end
-    io.stderr:write("\n["..category.."] ", utilities.concat(inputs, " "))
+    local message = utilities.concat(inputs, " ")
+    if message then io.stderr:write(("\n[%s] %s"):format(category, message)) end
   end
 end
 
@@ -120,21 +126,34 @@ utilities.debugAST = function (ast, level)
   end
   local out = string.rep("  ", 1+level)
   if level == 0 then
-    SU.debug("ast", "["..SILE.currentlyProcessingFile)
+    SU.debug("ast", function ()
+      return "[" .. SILE.currentlyProcessingFile
+    end)
   end
   if type(ast) == "function" then
-    SU.debug("ast", out .. tostring(ast))
-  end
-  for _, content in ipairs(ast) do
-    if type(content) == "string" then
-      SU.debug("ast", out .. "[" .. content .. "]")
-    elseif SILE.Commands[content.command] then
-      SU.debug("ast", out.."\\" .. content.command .. " " .. pl.pretty.write(content.options, ""))
-      if (#content>=1) then utilities.debugAST(content, level+1) end
-    elseif content.id == "texlike_stuff" or (not content.command and not content.id) then
-      utilities.debugAST(content, level+1)
-    else
-      SU.debug("ast", out.."?\\"..(content.command or content.id))
+    SU.debug("ast", function ()
+      return out .. tostring(ast)
+    end)
+  elseif type(ast) == "table" then
+    for _, content in ipairs(ast) do
+      if type(content) == "string" then
+        SU.debug("ast", function ()
+          return out .. "[" .. content .. "]"
+        end)
+      elseif type(content) == "table" then
+        if SILE.Commands[content.command] then
+          SU.debug("ast", function ()
+            return out .. "\\" .. content.command .. " " .. pl.pretty.write(content.options, "")
+          end)
+          if (#content>=1) then utilities.debugAST(content, level+1) end
+        elseif content.id == "texlike_stuff" or (not content.command and not content.id) then
+          utilities.debugAST(content, level+1)
+        else
+          SU.debug("ast", function ()
+            return out .. "?\\" .. (content.command or content.id)
+          end)
+        end
+      end
     end
   end
   if level == 0 then SU.debug("ast", "]") end
@@ -284,11 +303,6 @@ utilities.cast = function (wantedType, value)
   wantedType = string.lower(wantedType)
   if wantedType:match(actualType)     then return value
   elseif actualType == "nil" and wantedType:match("nil") then return nil
-  elseif wantedType:match("integer") or wantedType:match("number") then
-    if type(value) == "table" and type(value.tonumber) == "function" then
-      return value:tonumber()
-    end
-    return tonumber(value)
   elseif wantedType:match("length")      then return SILE.length(value)
   elseif wantedType:match("measurement") then return SILE.measurement(value)
   elseif wantedType:match("vglue")       then return SILE.nodefactory.vglue(value)
@@ -297,6 +311,27 @@ utilities.cast = function (wantedType, value)
   elseif actualType == "nil" then SU.error("Cannot cast nil to " .. wantedType)
   elseif wantedType:match("boolean")     then return SU.boolean(value)
   elseif wantedType:match("string")      then return tostring(value)
+  elseif wantedType:match("number") then
+    if type(value) == "table" and type(value.tonumber) == "function" then
+      return value:tonumber()
+    end
+    local num = tonumber(value)
+    if not num then SU.error("Cannot cast '" .. value .. "'' to " .. wantedType) end
+    return num
+  elseif wantedType:match("integer") then
+    local num
+    if type(value) == "table" and type(value.tonumber) == "function" then
+      num = value:tonumber()
+    else
+      num = tonumber(value)
+    end
+    if not num then SU.error("Cannot cast '" .. value .. "'' to " .. wantedType) end
+    if not wantedType:match("number") and num % 1 ~= 0 then
+      -- Could be an error but since it wasn't checked before, let's just warn:
+      -- Some packages might have wrongly typed settings, for instance.
+      SU.warn("Casting an integer but got a float number " .. num)
+    end
+    return num
   else SU.error("Cannot cast to unrecognized type " .. wantedType)
   end
 end
@@ -313,6 +348,11 @@ utilities.contentToString = function (content)
     if type(content[i]) == "table" and type(content[i][1]) == "string" then
       string = string .. content[i][1]
     elseif type(content[i]) == "string" then
+      -- Work around PEG parser returning env tags as content
+      -- TODO: refactor capture groups in PEG parser
+      if content.command == content[i] and content[i] == content[i+1] then
+        break
+      end
       string = string .. content[i]
     end
   end
@@ -343,6 +383,29 @@ utilities.walkContent = function (content, action)
   end
 end
 
+--- Strip position, line and column recursively from a content tree.
+-- This can be used to remove position details where we do not want them,
+-- e.g. in table of contents entries (referring to the original content,
+-- regardless where it was exactly, for the purpose of checking whether
+-- the table of contents changed.)
+--
+utilities.stripContentPos = function (content)
+  if type(content) ~= "table" then
+    return content
+  end
+  local stripped = {}
+  for k, v in pairs(content) do
+    if type(v) == "table" then
+      v = SU.stripContentPos(v)
+    end
+    stripped[k] = v
+  end
+  if content.id or content.command then
+    stripped.pos, stripped.col, stripped.lno = nil, nil, nil
+  end
+  return stripped
+end
+
 utilities.rateBadness = function(inf_bad, shortfall, spring)
   if spring == 0 then return inf_bad end
   local bad = math.floor(100 * math.abs(shortfall / spring) ^ 3)
@@ -521,60 +584,6 @@ end
 utilities.utf16be_to_utf8 = function (str) return utf16_to_utf8(str, "be") end
 utilities.utf16le_to_utf8 = function (str) return utf16_to_utf8(str, "le") end
 
-local icu = require("justenoughicu")
-
-local icuFormat = function (num, format)
-  local ok, result  = pcall(icu.format_number, num, format)
-  return tostring(ok and result or num)
-end
-
--- Language specific number formatters add functions to this table,
--- see e.g. languages/tr.lua
-utilities.formatNumber = {
-  und = {
-
-    alpha = function (num)
-      local out = ""
-      local a = string.byte("a")
-      repeat
-        num = num - 1
-        out = string.char(num % 26 + a) .. out
-        num = (num - num % 26) / 26
-      until num < 1
-      return out
-    end
-
-  }
-}
-
-setmetatable (utilities.formatNumber, {
-    __call = function (self, num, format, case)
-      if math.abs(num) > 9223372036854775807 then
-        SU.warn("Integers larger than 64 bits do not reproduce properly in all formats")
-      end
-      if not case then
-        if format:match("^%l") then
-          case = "lower"
-        elseif format:match("^.%l") then
-          case = "title"
-        else
-          case = "upper"
-        end
-      end
-      local lang = format:match("[Rr][Oo][Mm][Aa][Nn]") and "la" or SILE.settings:get("document.language")
-      format = format:lower()
-      local result
-      if self[lang] and type(self[lang][format]) == "function" then
-        result = self[lang][format](num)
-      elseif type(self["und"][format]) == "function" then
-        result = self.und[format](num)
-      else
-        result = icuFormat(num, format)
-      end
-      return icu.case(result, lang, case)
-    end
-})
-
 utilities.breadcrumbs = function ()
   local breadcrumbs = {}
 
@@ -596,7 +605,8 @@ utilities.breadcrumbs = function ()
   end
 
   function breadcrumbs:parent (count)
-    return self[#self-(count or 1)]
+    -- Note LuaJIT does not support __len, so this has to work even when that metamethod doesn't fire...
+    return self[#SILE.traceStack-(count or 1)]
   end
 
   function breadcrumbs:contains (needle)
@@ -609,4 +619,8 @@ utilities.breadcrumbs = function ()
   return breadcrumbs
 end
 
+utilities.formatNumber = require("core.utilities-numbers")
+
+utilities.collatedSort = require("core.utilities-sorting")
+
 return utilities
