@@ -29,8 +29,8 @@ package.documentation = [[
 SILE is Unicode compatible, and expects its input files to be in the UTF-8 encoding.
 (The actual range of Unicode characters supported will depend on the supported ranges of the fonts that SILE is using to typeset.)
 Some Unicode characters are hard to locate on a standard keyboard, and so are difficult to enter into SILE documents.
-The \autodoc:package{unichar} package helps with this problem by providing a command to enter Unicode codepoints.
-After loading \autodoc:package{unichar}, the \autodoc:command{\unichar} command becomes available:
+
+The \autodoc:package{unichar} package helps with this problem by providing the \autodoc:command{\unichar} command to enter Unicode codepoints.
 
 \begin[type=autodoc:codeblock]{raw}
 \unichar{U+263A}
